"""
  pyMusique - iTMS client

  Travis Watkins <alleykat@gmail.com>
  Cody Brocious <daeken@gmail.com>
  Charles M. Hannum <root@ihack.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  (C) Copyright 2005 Travis Watkins, Cody Brocious, Charles M. Hannum
"""

import os, sys

dir = os.path.split(sys.argv[0])[0]
prefix = os.path.split(dir)[0]
if prefix == '': prefix = '.'
sys.path = [ prefix + '/lib/pymusique' ] + sys.path

import md5
import qtstream
import config
from debug import *

try:
    import mcrypt
except ImportError:
    print 'mcrypt module not available, exiting'
    sys.exit(1)

NO_ERROR = 0
NOT_ENCRYPTED = 1
MISSING_KEY = 2
INVALID_KEY = 4

class DRMS:
    drms_data = None
    stsd_data = None
    meta_data = None
    error = 0
    sbuffer = ''

    def __init__(self, f):
        containers = ('moov', 'trak', 'udta', 'mdia', 'minf', 'dinf', 'stbl',
                      'ilst', '\251alb', '\251ART', '\251day', '\251nam', '\251cmt',
                      '\251too', '\251wrt', '----', 'sinf', 'SINF', 'schi')
        special = {'stsd':self.stsd, 'drms':self.drms, 'mp4a':self.drms, 'meta':self.meta}
        self.q = qtstream.QTStream(f, containers, special)
        self.DeDRMS()

    def DeDRMS(self):
        q = self.q
        uint32 = qtstream.uint32
        try:
            ad_user = q.getAtom('user')[0][1]
            ad_key  = q.getAtom('key ')[0][1]
            ad_iviv = q.getAtom('iviv')[0][1]
            strname = q.getAtom('name')[0][1]
            id_priv, ad_priv = q.getAtom('priv')[0]
        except IndexError:
            Debug(WARNING, 'drm', 'File is not encrypted.')
            self.error = NOT_ENCRYPTED
            return False

        userid, keyid = uint32(ad_user[:4]), uint32(ad_key[:4])
        if userid != 0 or keyid != 0:
            try:
                key = file(config.drmsdir + '/%08X.%03i' % (userid, keyid), 'r').read()
            except IOError:
                Debug(ERROR, 'drm', 'Key not found.')
                self.error = MISSING_KEY
                return False
        else:
            key = 'tr1-th3n.y00_by3'

        m = mcrypt.MCRYPT('rijndael-128', 'cbc')
        m.init(key, md5.new(strname[:strname.find('\0')] + ad_iviv).digest())
        sz = len(ad_priv)
        sz -= sz % 16
        ad_priv = m.decrypt(ad_priv[:sz]) + ad_priv[sz:]
        if ad_priv[:4] != 'itun':
            debug.Debug(debug.drm, 'Invalid key.')
            self.error = INVALID_KEY
            return False
        q.setAtom(id_priv, 'priv', ad_priv)

        p = qtstream.QTStream(ad_priv[4:], (), ())
        key, iv = p.getAtom('key ')[0][1], p.getAtom('iviv')[0][1]

        m = mcrypt.MCRYPT('rijndael-128', 'cbc')
        m.init(key, iv)

        Debug(INFO, 'drm', 'Decrypting audio.')

        ad_stsz = q.getAtom('stsz')[0][1]
        ad_stsc = q.getAtom('stsc')[0][1]
        id_stco, ad_stco = q.getAtom('stco')[0]
        mdat = q.getAtom('mdat')
        mdatpos = q.getAtomPos('mdat')

        stszoff = 12
        stscmax = 12 * uint32(ad_stsc[4:8])
        for stscoff in xrange(8, 8 + stscmax, 12):
            sc = ad_stsc[stscoff:stscoff + 12]
            firstchunk = uint32(sc[0:4]) - 1
            nsamps = uint32(sc[4:8])
            if stscoff == stscmax - 4:
                lastchunk = uint32(ad_stco[4:8])
            else:
                lastchunk = uint32(ad_stsc[stscoff + 12:stscoff + 16]) - 1
            #print "stscoff", stscoff, "firstchunk", firstchunk, "nsamps", nsamps, "lastchunk", lastchunk
            for stcooff in xrange(8 + 4 * firstchunk, 8 + 4 * lastchunk, 4):
                fileoff = uint32(ad_stco[stcooff:stcooff + 4])
                #print "stcooff", stcooff, "fileoff", fileoff
                for sect in xrange(len(mdat)):
                    if fileoff >= mdatpos[sect] and fileoff < (mdatpos[sect] + len(mdat[sect][1])):
                        break
                mdatoff = fileoff - mdatpos[sect]
                #print "sect", sect, "mdatoff", mdatoff, "len", len(mdat[sect][1])
                data = mdat[sect][1][:mdatoff]
                for k in xrange(nsamps):
                    sz = uint32(ad_stsz[stszoff:stszoff + 4])
                    stszoff += 4
                    samp = mdat[sect][1][mdatoff:mdatoff + sz]
                    mdatoff += sz
                    sz -= sz % 16
                    m.reinit()
                    data += m.decrypt(samp[:sz]) + samp[sz:]
                data += mdat[sect][1][mdatoff:]
                mdat[sect][1] = data
                #print "sect", sect, "mdatoff", mdatoff, "len", len(mdat[sect][1])

        Debug(INFO, 'drm', 'Finished.')

        for sect in xrange(len(mdat)):
            q.setAtom(mdat[sect][0], 'mdat', mdat[sect][1])
        id_drms, ad_drms = q.getAtom('drms')[0]
        q.setAtom(id_drms, 'mp4a', ad_drms)
        #for x in ('sinf', 'akID', 'apID', 'atID', 'cnID', 'geID', 'plID', 'rtng', 'xeID'): q.delAtom(x)
        #q.printTree()
        # q.delAtom(''

        newmdatpos = q.getAtomPos('mdat')
        #print "mdatpos=", mdatpos, "newmdatpos=", newmdatpos
        data = ad_stco[:8]
        for i in xrange(8, 8 + 4 * uint32(ad_stco[4:8]), 4):
            data += uint32(uint32(ad_stco[i:i + 4]) - mdatpos[0] + newmdatpos[0])
        data += ad_stco[len(data):]
        q.setAtom(id_stco, 'stco', data)

    def stsd(self, q, data, enc=0):
        if not enc:
            self.stsd_data = data[:8]
            return q.getAtomTree(data[8:])
        elif enc == 1:
            return self.stsd_data + q.getData(data)
        else:
            return len(self.stsd_data) + len(q.getData(data))

    def drms(self, q, data, enc=0):
        if not enc:
            self.drms_data = data[:28]
            return q.getAtomTree(data[28:])
        elif enc == 1:
            return self.drms_data + q.getData(data)
        else:
            return len(self.drms_data) + len(q.getData(data))

    def meta(self, q, data, enc=0):
        if not enc:
            self.meta_data = data[:4]
            return q.getAtomTree(data[4:])
        elif enc == 1:
            return self.meta_data + q.getData(data)
        else:
            return len(self.meta_data) + len(q.getData(data))
