"""
  PyMusique - iTMS client

  Jon Lech Johansen <jon@nanocrew.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  (C) Copyright 2005 Jon Lech Johansen
"""

usegst = False
usevlc = False

try:
    import vlc
    usevlc = True
except ImportError:
    import gst
    usegst = True

class Player:
    def __init__(self):
        global usevlc, usegst
        if usevlc:
            self.vid = vlc.create()
            vlc.init(self.vid)
            self.supportsM4P = True
        elif usegst:
            self.playbin = gst.element_factory_make('playbin')
            try:
                import gconf
                client = gconf.client_get_default()
                sinkname = client.get_string('/system/gstreamer/0.8/default/audiosink')
                self.audiosink = gst.parse_launch(sinkname)
            except ImportError:
                self.audiosink = None
            self.supportsM4P = False

    def playURI(self, uri):
        global usevlc, usegst
        if usevlc:
            vlc.stop(self.vid)
            vlc.addTarget(self.vid, uri)
            vlc.play(self.vid)
            return True
        elif usegst:
            self.playbin.set_state(gst.STATE_NULL)

            self.playbin.set_property('uri', uri)

            if self.audiosink:
                self.playbin.set_property('audio-sink', self.audiosink)

            if not self.playbin.set_state(gst.STATE_PLAYING):
                return False

            return True

    def playFile(self, filename):
        global usevlc, usegst
        if usevlc:
            return self.playURI(filename)
        elif usegst:
            return self.playURI('file://%s' % filename)

    def stop(self):
        global usevlc, usegst
        if usevlc:
            vlc.stop(self.vid)
        elif usegst:
            self.playbin.set_state(gst.STATE_NULL)
