"""
  PyMusique - iTMS client

  Travis Watkins <alleykat@gmail.com>
  Cody Brocious <daeken@gmail.com>
  Jon Lech Johansen <jon@nanocrew.net>
  Charles M. Hannum <root@ihack.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

  (C) Copyright 2005 Travis Watkins, Cody Brocious, Jon Lech Johansen,
  Charles M. Hannum
"""

import struct
import libxml2
import urllib
from binascii import a2b_hex
from storebase import *
from debug import *

iv = a2b_hex('24e6f543f3871056ab34f27f65ff0076')

def ProcessRawM4A(data, key):
    if len(key) == 32:
        key = a2b_hex(key)
    elif len(key) != 16:
        return False
    m = mcrypt.MCRYPT('rijndael-128', 'cbc')
    m.init(key, iv)
    data = m.decrypt(data)
    trailer = ord(data[-1])
    if trailer:
        data = data[:-trailer]
    return data

class Store(StoreBase):
    balance = None
    dspbalance = None

    def SignupStep1(self, params, callback):
        self.signup_params = params
        self.signup_callback = callback
        self.Request('signup', self.SignupStep1Sub1, 'guid=' + self.guid)

    def GetGotoURL(self, doc, name, value):
        items = doc.xpathEval('//*[name()="GotoURL"]')
        if not items:
            return None
        for item in items:
            for element in item.get_children():
                while element:
                    attr = element.get_properties()
                    while attr:
                        if attr.get_name() == name and attr.get_content() == value:
                            attr2 = item.get_properties()
                            while attr2:
                                if attr2.get_name() == 'url':
                                    return attr2.get_content()
                                attr2 = attr2.get_next()
                        attr = attr.get_next()
                    element = element.next
        return None

    def GetCCs(self, doc):
        ccs = {}
        items = doc.xpathEval('//*[name()="GotoURL"]')
        if not items:
            return None
        for item in items:
            for element in item.get_children():
                while element:
                    attr = element.get_properties()
                    while attr:
                        if attr.get_name() == 'url' and '/cc_' in attr.get_content():
                            attr2 = item.get_properties()
                            while attr2:
                                if attr2.get_name() == 'url':
                                    cc = re.search('/cc_(.*?).png', attr.get_content()).group(1)
                                    ccs[cc] = attr2.get_content()
                                attr2 = attr2.get_next()
                        attr = attr.get_next()
                    element = element.next
        return ccs

    def GetRedText(self, doc):
        redtext = ''
        items = doc.xpathEval('//*[name()="TextView" and @normalStyle="lucida13Red"]')
        if not items:
            return None
        for item in items:
            redtext = redtext + item.get_content()
        if len(redtext) > 1:
            return redtext
        return None

    def SignupStep1Sub1(self, data):
        doc = libxml2.parseDoc(data[1])
        nexturl = self.GetGotoURL(doc, 'viewName', 'agreeButton')
        if nexturl == None or not 'set-cookie' in data[0]:
            self.signup_callback(False, None)
            return
        cookies = data[0]['set-cookie'][0] + ';' + data[0]['set-cookie'][1]
        wosid = re.search('wosid=(.*?);', cookies).group(1)
        woinst = re.search('woinst=(.*?);', cookies).group(1)
        self.cookies = 'woinst=%s; wosid=%s' % (woinst, wosid)
        self.Request('https://phobos.apple.com' + nexturl, self.SignupStep1Sub2, '', self.cookies)

    def SignupStep1Sub2(self, data):
        doc = libxml2.parseDoc(data[1])
        nexturl = self.GetGotoURL(doc, 'viewName', 'continueButton')
        if nexturl == None:
            self.signup_callback(False, None)
            return

        formdata = { 'continueButton':'submit',
                     'birthDay':'1',
                     'birthMonth':'1',
                     'acceptsNewsletter':'0',
                     'acceptsMarketingEmail':'0' }
        for key in self.signup_params.keys():
            formdata[key] = self.signup_params[key]

        self.Request('https://phobos.apple.com' + nexturl, self.SignupStep1Sub3, '', self.cookies, None, urllib.urlencode(formdata))

    def SignupStep1Sub3(self, data):
        doc = libxml2.parseDoc(data[1])
        nexturl = self.GetGotoURL(doc, 'viewName', 'doneButton')
        if nexturl == None:
            self.signup_callback(False, self.GetRedText(doc))
            return
        self.signup_nexturl = self.GetCCs(doc)
        self.signup_callback(True, self.signup_nexturl.keys())

    def SignupStep2(self, cc, params, callback):
        self.signup_params = params
        self.signup_callback = callback
        nexturl = self.signup_nexturl[cc]
        self.Request('https://phobos.apple.com' + nexturl, self.SignupStep2Sub, '', self.cookies)

    def SignupStep2Sub(self, data):
        doc = libxml2.parseDoc(data[1])
        nexturl = self.GetGotoURL(doc, 'viewName', 'doneButton')
        if nexturl == None:
            self.signup_callback(False, None)
            return

        formdata = { 'doneButton':'submit',
                     'prefixName':'Dr',
                     'street2':'' }
        for key in self.signup_params.keys():
            formdata[key] = self.signup_params[key]

        self.Request('https://phobos.apple.com' + nexturl, self.SignupStep2End, '', self.cookies, None, urllib.urlencode(formdata))

    def SignupStep2End(self, data):
        if 'signupSuccess' in data[1]:
            self.signup_callback(True, None)
            return
        doc = libxml2.parseDoc(data[1])
        self.signup_callback(False, self.GetRedText(doc))

    def Login(self, user, password, callback):
        self.login_callback = callback
        self.Request('authenticateAccount', self.LoginEnd, 'appleId=' + urllib.quote(user) + '&password=' + urllib.quote(password) + '&accountKind=0&attempt=1&guid=' + self.guid + '&signIn=true&createSession=true')

    def LoginEnd(self, data):
        headers = data[0]
        dp = DictParse(data[1])
        if not dp.has_key('jingleDocType') or dp['jingleDocType'] != 'authenticationSuccess':
            self.login_callback(False)
            return
        self.dsid, self.token = dp['dsPersonId'], dp['passwordToken']
        self.balance = int(dp['creditBalance'])
        self.dspbalance = dp['creditDisplay']
        if dp.has_key('freeSongBalance'):
            self.fsbalance = dp['freeSongBalance']
        else:
            self.fsbalance = 0
        self.login_callback(True)

    def PendingSongs(self, callback):
        self.pending_callback = callback
        self.Request('pendingSongs', self.PendingEnd, 'guid=' + self.guid)

    def PendingEnd(self, data):
        self.pending_callback(DictParse(data[1])['songList'])

    def Authorize(self):
        dp = DictParse(self.Request('authorizeMachine', 'guid=' + self.guid)[1])
        if dp['jingleDocType'] == 'machineAuthorizationInfoSuccess':
            return dp['encryptionKeys']
        return False

    def Deauthorize(self):
        dp = DictParse(self.Request('deauthorizeMachine', 'guid=' + self.guid)[1])
        if dp['jingleDocType'] == 'success':
            return True
        return False

    def GetAuthInfo(self):
        dp = DictParse(self.Request('machineAuthorizationInfo', 'guid=' + self.guid)[1])
        if dp['jingleDocType'] == 'machineAuthorizationInfoSuccess':
            return dp
        return False

    def Search(self, type, search, callback):
        self.search_callback = callback
        if type == 'all':
            self.Request('search', self.EndSearch, 'term=' + urllib.quote(search))

    def EndSearch(self, data):
        xml = DictParse(data[1])
        albums, songs = [], []
        if 'ping' in xml:
            xml = xml['ping']
        for song in xml['items']:
            songs.append(song)
        self.search_callback((albums, songs))

    def Buy(self, buyparams, callback):
        self.buy_callback = callback
        params = buyparams + '&creditBalance=' + str(self.balance) + '&creditDisplay=' + urllib.quote(self.dspbalance) + '&freeSongBalance=' + str(self.fsbalance) + '&guid=' + self.guid + '&rebuy=false&buyWithoutAuthorization=true&wasWarnedAboutFirstTimeBuy=true'
        self.Request('buyProduct', self.BuyEnd, params)

    def BuyEnd(self, data):
        xml = DictParse(data[1])
        if not xml.has_key('jingleDocType') or xml['jingleDocType'] != 'purchaseSuccess':
            self.buy_callback(False, xml['customerMessage'])
        else:
            self.buy_callback(xml['songList'])
